// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

namespace {
  class DeleteObjectFactory
    : public Factory
  {
  public:
    DeleteObjectFactory ()
      : Factory ("DeleteObject")
      {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (ActionConfiguration ());
    }
    SP<Configurable> makeIt (const SP<Configuration> c) const {
      SP<ActionConfiguration> conf =
	dynamic_cast <ActionConfiguration *> (&*c);
      assert (conf);
      SP<TopLevel> top = conf->getTopLevel();
      int index;
      Vec3 intersectionPoint;
      SP<Action> result = NEW(Action());
      SP<SceneGraph> sg = top->getSceneGraph();
      if (top->findUnderMouse (index, intersectionPoint)) {
	result->setMessage (FString ("Deleting object ")+index);
	sg->deleteObject (index);
      } else {
	result->setProblem ("Delete was pressed when the mouse was "
			    "not over an object");
      }
      return &*result;
    }
  };
  const bool useless =
  (FactoryTable::store ("KeyboardMouseAction", NEW (DeleteObjectFactory ())),
   true);
} // namespace
