// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef TopConfiguration_h_included
#include "TopConfiguration.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef SceneGraphConfiguration_h_included
#include "SceneGraphConfiguration.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

namespace {
  class NewSceneFactory
    : public Factory
  {
  public:
    NewSceneFactory ()
      : Factory ("NewScene")
      {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (ActionConfiguration ());
    }
    SP<Configurable> makeIt (const SP<Configuration> c) const {
      SP<ActionConfiguration> conf =
	dynamic_cast <ActionConfiguration *> (&*c);
      assert (conf);
      SP<TopConfiguration> top = conf->getTopLevel()->getConfiguration();
      SP<SceneGraphConfiguration> sgc =
	dynamic_cast <SceneGraphConfiguration *>
	(top->getSceneGraphSlot()->getConfiguration());
      assert (sgc);
      sgc->setSceneGraph (0);
      SP<SceneGraph> sg =
	dynamic_cast <SceneGraph *> (&*top->getSceneGraphSlot()->makeIt());
      assert (sg);
#ifndef NDEBUG
      // We need the old SceneGraph to go away promptly so the
      // SelectionManager will see that its object links are invalid and make
      // new ones that point into the new SceneGraph.
      SP<SceneGraph> oldSg = conf->getTopLevel()->getSceneGraph ();
#endif
      conf->getTopLevel()->setSceneGraph (sg);
#ifndef NDEBUG
      if (oldSg) {
	// The only reference to the old SceneGraph should be the one we're
	// about to clobber.
	assert (1 == oldSg->count());
	oldSg = 0;
      }
#endif
      SP<RecursiveSlotValue> rs = top->getSceneLoaderSlot ();
      assert (rs);
      SP<ActionConfiguration> ac =
	dynamic_cast <ActionConfiguration *> (rs->getConfiguration ());
      assert (ac);
      ac->setTopLevel (conf->getTopLevel());
      // Next step actually creates the scene.
      SP<Configurable> slc = rs->makeIt ();
      assert (slc);
      // Avoid reference cycle and thus memory leak.
      // FIXME I do not think this is needed any more.
      // ac->setTopLevel (0);
      return slc;
    }
  };
  const bool useless =
  (FactoryTable::store ("Action", NEW (NewSceneFactory ())),
   true);
} // namespace
