// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef TransformConfiguration_h_included
#include "TransformConfiguration.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef Vec3SlotValue_h_included
#include "Vec3SlotValue.h"
#endif

#ifndef QuaternionSlotValue_h_included
#include "QuaternionSlotValue.h"
#endif

#ifndef Matrix4_h_included
#include "Matrix4.h"
#endif

#ifndef Quaternion_h_included
#include "Quaternion.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Canvas_h_included
#include "Canvas.h"
#endif

#ifndef CursorBallManager_h_included
#include "CursorBallManager.h"
#endif

#ifndef SpaceOrbEvent_h_included
#include "SpaceOrbEvent.h"
#endif

#ifndef CursorBall_h_included
#include "CursorBall.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

namespace {
  FString translationName () {
    return "Translation";
  }
  FString centerName () {
    return "Center of rotation";
  }
  FString rotationName () {
    return "Rotation";
  }
}

TransformConfiguration::TransformConfiguration () {
  initializeSlot (translationName (), NEW (Vec3SlotValue (Vec3 (0,0,0))));
  initializeSlot (centerName (), NEW (Vec3SlotValue (Vec3 (0, 0, 0))));
  initializeSlot (rotationName (), NEW (QuaternionSlotValue (Quaternion ())));
}

Vec3 TransformConfiguration::getTranslation () const {
  return Vec3SlotValue::getSlot (this, translationName());
}

void TransformConfiguration::setTranslation (const Vec3 &v) {
  Vec3SlotValue::setSlot (this, translationName(), v);
}

Vec3 TransformConfiguration::getCenter () const {
  return Vec3SlotValue::getSlot (this, centerName ());
}

void TransformConfiguration::setCenter (const Vec3 &v) {
  Vec3SlotValue::setSlot (this, centerName(), v);
}

Quaternion TransformConfiguration::getRotation () const {
  return QuaternionSlotValue::getSlot (this, rotationName ());
}

void TransformConfiguration::setRotation (const Quaternion &v) {
  QuaternionSlotValue::setSlot (this, rotationName(), v);
}

Matrix4 TransformConfiguration::getMatrix () {
  Vec3 center = getCenter ();
  // Read this from right to left.
  return Matrix4::translate (getTranslation ()) * Matrix4::translate (center)
    * getRotation () * Matrix4::translate (-center);
}

void TransformConfiguration::parseOrbEvent (const SpaceOrbEvent *e) {
  SP<TopLevel> top = getTopLevel ();
  assert (top);
  SP<Canvas> canvas = top->getCanvas ();
  setTranslation (e->getTranslation () * canvas->getScale ());
  setCenter (canvas->transformWorldToRHScreen
	     (CursorBallManager::getCursorBall
	      (top->getSceneGraph ())->getPosition ()));
  setRotation (e->getRotation());
}

bool TransformConfiguration::hasTheSameTypeAs (const Configuration *c) const {
  return 0 != dynamic_cast <const TransformConfiguration *> (c);
}

SP<Configuration> TransformConfiguration::copy () const {
  return NEW (TransformConfiguration (*this));
}
