// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef BoringAtom_h_included
#define BoringAtom_h_included

#ifndef DDPO_h_included
#include "DDPO.h"
#endif

class FString;
class AtomConfiguration;
class RecursiveSlotValue;
class Factory;
class BoringAtom
  : public DDPO
{
  struct BoringAtomData;
  BoringAtomData *m_data;
  class BoringAtomFactory;
  friend class BoringAtomFactory;
protected:
  // The constructor of BoringAtom takes a factory so we can implement the
  // getFactory method.  See the comment for that method below.
  BoringAtom (AtomConfiguration *b, const Factory *f);
public:
  ~BoringAtom ();
  // If you want the symbol name, get the symbol number and then use
  // AtomInfo::toSymbol.
  int getSymbolNumber () const;
  // How this atom was made.  If you want to make a similar atom, call
  // getFactory, make a copy, edit the copy, and call makeIt on the copy.
  SP<const RecursiveSlotValue> getFactory () const;
  static const char *const staticClassId;
  const char *classId () const;
  bool isInstanceOf (const char *classId) const;
  // Will return 6: 3 for state, then 3 for velocity.
  int stateSize () const;
  void plausibleState (Dynavec <FFloat> &result) const;
  static Vec3 velocity (const FFloat *state);
  static void setVelocity (FFloat *state, const Vec3 &v);
  static Vec3 center (const FFloat *state);
  static void setCenter (FFloat *state, const Vec3 &v);
  // We have to implement a bunch of boring PhysicsObject methods to make it
  // possible to instantiate a BoringAtom. These atoms are boring, so they
  // don't do interesting physics.
  // physicsIntersect always returns false.
  bool physicsIntersect (const FFloat *state,
			 const BoundingBox &box) const;
  // The boundingSphereCenter is the position of hte atom.
  Vec3 boundingSphereCenter (const FFloat *state) const;
  // the physicsBoundingSphereRadius is zero.
  FFloat physicsBoundingSphereRadius () const;
  // isPhysicsBounded returns true.
  bool isPhysicsBounded () const;
  // translate translates the position.
  void translate (FFloat *state, const Vec3 &v) const;
  void addVelocity (FFloat *state, const Vec3 &v) const;
  // rotate does nothing.
  void rotate (FFloat *state, const Quaternion &q) const;
  // exempt always returns true.
  bool exempt () const;
};
#endif
