// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef ChangeAtomTypeConfiguration_h_included
#include "ChangeAtomTypeConfiguration.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

namespace {
  FString atomFactoryName () {
    return "Factory for replacement atoms";
  }
}

ChangeAtomTypeConfiguration::ChangeAtomTypeConfiguration () {
  initializeSlot (atomFactoryName (),
		  NEW (RecursiveSlotValue ("Atom", "BoringAtom")));
}

SP<Configuration> ChangeAtomTypeConfiguration::copy () const {
  // Default copy constructor is good.
  return NEW (ChangeAtomTypeConfiguration (*this));
}

bool ChangeAtomTypeConfiguration::hasTheSameTypeAs (const Configuration *c) const {
  return 0 != dynamic_cast <const ChangeAtomTypeConfiguration *> (c);
}

// The user can specify the factory for making replacement atoms.
SP<RecursiveSlotValue> ChangeAtomTypeConfiguration::getAtomSlot () {
  return RecursiveSlotValue::getSlotValue (this, atomFactoryName ());
}

void ChangeAtomTypeConfiguration::setAtomSlot (SP<RecursiveSlotValue> rsv) {
  RecursiveSlotValue::setSlotValue (this, atomFactoryName(), rsv);
}
