// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef StubBrennerAtom_h_included
#include "StubBrennerAtom.h"
#endif

#ifndef TypedFactory_h_included
#include "TypedFactory.h"
#endif

#ifndef StubBrennerAtomConfiguration_h_included
#include "StubBrennerAtomConfiguration.h"
#endif

#ifndef AtomInfo_h_included
#include "AtomInfo.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef AtomCache_h_included
#include "AtomCache.h"
#endif

typedef StubBrennerAtom::MotionFlag MotionFlag;

StubBrennerAtom::StubBrennerAtom
(StubBrennerAtomConfiguration *conf, const Factory *f)
  : BoringAtom (conf, f),
    m_flag (conf->getMotionFlag ())
{}

MotionFlag StubBrennerAtom::getMotionFlag () const {
  return m_flag;
}

const char *const StubBrennerAtom::staticClassId = "StubBrennerAtom";

const char *StubBrennerAtom::classId () const {
  return staticClassId;
}

bool StubBrennerAtom::isInstanceOf (const char *classId) const {
  return classId == staticClassId || BoringAtom::isInstanceOf (classId);
}

class StubBrennerAtom::StubBrennerAtomFactory
  : public TypedFactory <StubBrennerAtomConfiguration, StubBrennerAtom>
{
  static SP<AtomCache <StubBrennerAtom> > s_cache;
  static void deallocate () {
    if (s_cache) {
      s_cache = 0;
    }
  }
  static const bool s_useless;
public:
  StubBrennerAtomFactory ()
    : TypedFactory <StubBrennerAtomConfiguration, StubBrennerAtom>
  ("StubBrennerAtom")
  {}
  SP<StubBrennerAtomConfiguration> typedDefaultConfiguration () const {
    return NEW (StubBrennerAtomConfiguration ());
  }
  SP<StubBrennerAtom> makeIt (StubBrennerAtomConfiguration *bac) const {
    SP<StubBrennerAtom> &ba = s_cache->cacheref (bac);
    if (!ba) {
      ba = NEW (StubBrennerAtom (bac, this));
    }
    return ba;
  }
};

SP<AtomCache<StubBrennerAtom> >
StubBrennerAtom::StubBrennerAtomFactory::s_cache =
NEW (AtomCache <StubBrennerAtom> ());

const bool StubBrennerAtom::StubBrennerAtomFactory::s_useless =
(FactoryTable::store ("Atom", NEW (StubBrennerAtomFactory())),
 MemoryUtil::registerDeallocator (deallocate),
 true);
