// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 N Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef StubBrennerAtom_h_included
#define StubBrennerAtom_h_included

#ifndef BoringAtom_h_included
#include "BoringAtom.h"
#endif

#ifndef StubBrennerAtomConfiguration_h_included
#include "StubBrennerAtomConfiguration.h"
#endif

class Factory;
class StubBrennerAtom
  : public BoringAtom
{
  // StubBrennerAtomFactory is declared as a friend so we can declare the
  // StubBrennerAtom constructor as private, and then the compiler enforces the
  // constraint that only StubBrennerAtomFactory can make a StubBrennerAtom.
  // Another alternative would have been to declare this class inside a
  // namespace inside StubBrennerAtom.cpp, and not to provide a header file at
  // all.  This would have hidden the getMotionFlag method, which is required
  // by DSceneWriter so we can write .d files.  This almost prevents
  // StubBrennerAtom from being entirely private.  However, BoringAtom now has
  // a GetFactory method that could have been used to get the motion flag.
  // FIXME Make DSceneWriter use BoringAtom::getFactory to get the motion flag
  // and get rid of this public header file.
  class StubBrennerAtomFactory;
  friend class StubBrennerAtomFactory;
public:
  typedef StubBrennerAtomConfiguration::MotionFlag MotionFlag;
private:
  MotionFlag m_flag;
  StubBrennerAtom (StubBrennerAtomConfiguration *c, const Factory *f);
public:
  MotionFlag getMotionFlag () const;
  static const char *const staticClassId;
  const char *classId () const;
  bool isInstanceOf (const char *classId) const;
};
#endif
