// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef InputDevicesFactory_h_included
#define InputDevicesFactory_h_included

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

class FString;
class InputDevice;

class InputDevicesFactory
  : public Factory
{
protected:
  InputDevicesFactory (const FString &name);
public:
  // Running the factory gives you an InputDevices, which is a list of all of
  // the input devices that the InputDevicesFactory found and was able to
  // open. 
  // Lists all of the device types that are inferior to the present one.  Try
  // the best device types first.
  virtual Dynavec <SP <InputDevicesFactory> > inferiorDeviceFactories () const
    = 0;
  // Find all of the registered instances of InputDevicesFactory that start
  // with any of the the given prefixes, determine a good order to run them,
  // run them, concatenate the results and return.
  static void openDevices (const Dynavec <FString> &prefixes,
			   Dynavec <SP <InputDevice> > &inputDevices);
};

#endif
