// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef SpaceOrbConfiguration_h_included
#include "SpaceOrbConfiguration.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef PositiveFloatSlotValue_h_included
#include "PositiveFloatSlotValue.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef BoolSlotValue_h_included
#include "BoolSlotValue.h"
#endif

#ifndef SpaceOrbEvent_h_included
#include "SpaceOrbEvent.h"
#endif

#ifndef Event_h_included
#include "Event.h"
#endif

#ifndef DispatcherConfiguration_h_included
#include "DispatcherConfiguration.h"
#endif

#ifndef Macro_h_included
#include "Macro.h"
#endif

#ifndef TransformMacro_h_included
#include "TransformMacro.h"
#endif

#ifndef AnyOrbMove_h_included
#include "AnyOrbMove.h"
#endif

namespace {
  FString transGainName () {
    return "Translational Gain";
  }
  FString rotGainName () {
    return "Rotational Gain";
  }
  FString transStickName () {
    return "Translational Stiction";
  }
  FString rotStickName () {
    return "Rotational Stiction";
  }
};

SpaceOrbConfiguration::SpaceOrbConfiguration ()
  : InputDeviceConfiguration ("SpaceOrbAction")
{
  ref ();
  initializeSlot (transGainName (), NEW (PositiveFloatSlotValue (0.5)));
  initializeSlot (rotGainName (), NEW (PositiveFloatSlotValue (0.2)));
  initializeSlot (transStickName (), NEW (PositiveFloatSlotValue (0.1)));
  initializeSlot (rotStickName (), NEW (PositiveFloatSlotValue (0.04)));
  SP<SpaceOrbEvent> soe = NEW (SpaceOrbEvent ());
  SP<DispatcherConfiguration> conf =
    NEW (DispatcherConfiguration (&*soe,
				  "SpaceOrbAction",
				  Dynavec <Event::Button> (SpaceOrbEvent::A,
							   SpaceOrbEvent::B)));
  conf->bindButton (SpaceOrbEvent::C, "GrowSelectionConnected",
		    SpaceOrbEvent::B);
  conf->bindButton (SpaceOrbEvent::D, "GrowSelectionNearby",
		    SpaceOrbEvent::B);
  conf->bindButton (SpaceOrbEvent::D, "CursorBallToCG");
  conf->bindButton (SpaceOrbEvent::F,
		    AnyOrbMove::mkOrbMove
		    (TransformMacro::mkMacro
		     (Dynavec <FString>
		      ("TransformSelectionRHScreen",
		       "TransformCursorBallRHScreen"))));
  {
    SP<RecursiveSlotValue> rsv =
      AnyOrbMove::mkOrbMove ("TransformCursorBallRHScreen");
    conf->bindButton (SpaceOrbEvent::MOTION, rsv);
    conf->bindButton (SpaceOrbEvent::MOTION, rsv, SpaceOrbEvent::B);
  }
  conf->bindButton (SpaceOrbEvent::C,
		    AnyOrbMove::mkOrbMove ("TransformWorldMove"));
  SP<RecursiveSlotValue> dispatcherRSV =
    NEW (RecursiveSlotValue ("SpaceOrbAction",
			     "SpaceOrbDispatcher"));
  dispatcherRSV->setConfiguration (&*conf);
  setDispatcher (dispatcherRSV);
  deref ();
}

bool SpaceOrbConfiguration::hasTheSameTypeAs (const Configuration *c) const
{
  return 0 != dynamic_cast <const SpaceOrbConfiguration *> (c);
}

SP<Configuration> SpaceOrbConfiguration::copy () const {
  // Default copy constructor is sufficient.
  return NEW (SpaceOrbConfiguration (*this));
}

FString SpaceOrbConfiguration::unParse () const {
  return "Space Orb Configuration";
}

void SpaceOrbConfiguration::setTranslationalGain (FFloat gain) {
  PositiveFloatSlotValue::setSlot (this, transGainName (), gain);
}

FFloat SpaceOrbConfiguration::getTranslationalGain () const {
  return PositiveFloatSlotValue::getSlot (this, transGainName ());
}

void SpaceOrbConfiguration::setRotationalGain (FFloat gain) {
  PositiveFloatSlotValue::setSlot (this, rotGainName (), gain);
}

FFloat SpaceOrbConfiguration::getRotationalGain () const {
  return PositiveFloatSlotValue::getSlot (this, rotGainName ());
}

void SpaceOrbConfiguration::setTranslationalStick (FFloat stick) {
  PositiveFloatSlotValue::setSlot (this, transStickName (), stick);
}

FFloat SpaceOrbConfiguration::getTranslationalStick () const {
  return PositiveFloatSlotValue::getSlot (this, transStickName ());
}

void SpaceOrbConfiguration::setRotationalStick (FFloat stick) {
  PositiveFloatSlotValue::setSlot (this, rotStickName (), stick);
}

FFloat SpaceOrbConfiguration::getRotationalStick () const {
  return PositiveFloatSlotValue::getSlot (this, rotStickName ());
}

