// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#include "Vec3.h"

// Next one defines std::ostream.
#ifndef iostream_h_included
#include <iostream>
#define iostream_h_included
#endif

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

std::ostream & operator <<(std::ostream &o, const Vec3 &v) {
  return o<<"("<<v[0]<<", "<<v[1]<<", "<<v[2]<<")";
}

Vec3::operator FString () const {
  return FString ("(")+values[0]+", "+values[1]+", "+values[2]+")";
}

#ifndef Vec3i_h_included
#include "Vec3i.h"
#endif

Vec3::Vec3 (const Vec3i &v) {
  values [0] = v [0];
  values [1] = v [1];
  values [2] = v [2];
}

#ifdef TEST
int main () {
  Vec3 a (1, 2, 3);
  Vec3 b (4, 5, 6);
  Vec3 c = a.cross (b);
  const FFloat epsilon = 0.0001;
  assert (-epsilon < a * c && a * c < epsilon);
  assert (-epsilon < b * c && b * c < epsilon);
  Vec3 d = a.cross (a);
  assert (epsilon > d.length());
  assert ("Testing assert at the end" && 0);
}
#endif

