// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef InputDevicesFactory_h_included
#include "InputDevicesFactory.h"
#endif

#ifndef SimpleConfiguration_h_included
#include "SimpleConfiguration.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef Configurable_h_included
#include "Configurable.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef InputDevices_h_included
#include "InputDevices.h"
#endif

#ifndef InputDevice_h_included
#include "InputDevice.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef LinuxJoystick_h_included
#include "LinuxJoystick.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

namespace {
  class LJInputDevices
    : public InputDevices
  {
  public:
    void findDevices (Dynavec <SP <InputDevice> > &inputDevices)
    {
      inputDevices.extendTo (0);
      Dynavec <FString> devNames = LinuxJoystick::joystickDeviceNames ();
      for (int i = 0; i < devNames.size (); i++) {
	SP<LinuxJoystick> lj =
	  LinuxJoystick::openJoystickDevice (devNames [i]);
	if (lj) {
	  inputDevices.push (&*lj);
	}
      }
    }
  };
  class LinuxJoystickFactory
    : public InputDevicesFactory
  {
  public:
    LinuxJoystickFactory ()
      : InputDevicesFactory ("LinuxJoystickFactory")
    {}
    Dynavec <SP <InputDevicesFactory> > inferiorDeviceFactories () const {
      return Dynavec <SP <InputDevicesFactory> > ();
    }
    SP<Configuration> defaultConfiguration () const {
      return NEW (SimpleConfiguration ());
    }
    SP<Configurable> makeIt (SP<Configuration> conf) const {
      (void) conf;
      return NEW (LJInputDevices ());
    }
  };
  static const bool useless =
  (FactoryTable::store ("InputDevicesFactory", NEW (LinuxJoystickFactory ())),
   true);
};
