// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//
#ifndef MouseDragBindingConfiguration_h_included
#include "MouseDragBindingConfiguration.h"
#endif

#ifndef ButtonSlotValue_h_included
#include "ButtonSlotValue.h"
#endif

#ifndef KeyboardMouseEvent_h_included
#include "KeyboardMouseEvent.h"
#endif

#ifndef RecursiveSlotValue_h_included
#include "RecursiveSlotValue.h"
#endif

#ifndef VectorSlotValue_h_included
#include "VectorSlotValue.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef ConfigurationSlotValue_h_included
#include "ConfigurationSlotValue.h"
#endif

#ifndef Event_h_included
#include "Event.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

namespace {
  FString shiftsName () {
    return "Shift buttons";
  }
  FString dragName () {
    return "The MouseDrag to invoke";
  }
}

static SP<VectorSlotValue> defaultVSV () {
  SP<const Event> e = NEW (KeyboardMouseEvent ());
  SP<ButtonSlotValue> bsv = NEW (ButtonSlotValue (e));
  return NEW (VectorSlotValue (bsv));
}

MouseDragBindingConfiguration::MouseDragBindingConfiguration ()
{
  initializeSlot (shiftsName(), defaultVSV ());
  initializeSlot (dragName (),
                  NEW (RecursiveSlotValue ("MouseDrag", "NullMouseDrag")));
}

// The default copy constructor is fine.

SP<Configuration> MouseDragBindingConfiguration::copy () const {
  return NEW (MouseDragBindingConfiguration (*this));
}

bool MouseDragBindingConfiguration::hasTheSameTypeAs (const Configuration *c) const {
  return 0 != dynamic_cast <const MouseDragBindingConfiguration *> (c);
}

SP<const VectorSlotValue> MouseDragBindingConfiguration::getShiftsVSV () const
{
  return VectorSlotValue::getSlotValue (this, shiftsName ());
}

SP<VectorSlotValue> MouseDragBindingConfiguration::getShiftsVSV () 
{
  return VectorSlotValue::getSlotValue (this, shiftsName ());
}

void MouseDragBindingConfiguration::setShiftsVSV
(SP<VectorSlotValue> vsv)
{
  VectorSlotValue::setSlotValue (this, shiftsName (), vsv);
}

Dynavec <Event::Button> MouseDragBindingConfiguration::getShifts () const {
  SP<const VectorSlotValue> vsv = getShiftsVSV ();
  Dynavec <Button> result;
  for (int i = 0; i < vsv->size(); i++) {
    SP<const SlotValue> bsv = (*vsv)[i];
    assert (dynamic_cast <const ButtonSlotValue *> (&*bsv));
    result.push (static_cast <const ButtonSlotValue *> (&*bsv)->getValue());
  }
  return result;
}

void MouseDragBindingConfiguration::setShifts (const Dynavec <Button> &v) {
  SP<const Event> e = NEW (KeyboardMouseEvent ());
  SP<VectorSlotValue> vsv = defaultVSV ();
  vsv->extendTo (v.size());
  for (int i = 0; i < v.size(); i++) {
    SP<ButtonSlotValue> bsv = NEW (ButtonSlotValue (e, v[i]));
    vsv->setValue (i, &*bsv);
  }
  setShiftsVSV (vsv);
}

void MouseDragBindingConfiguration::setShift (Button b) {
  setShifts (Dynavec <Button> (b));
}

SP<const RecursiveSlotValue> MouseDragBindingConfiguration::getMouseDrag () const {
  return RecursiveSlotValue::getSlotValue (this, dragName ());
}

SP<RecursiveSlotValue> MouseDragBindingConfiguration::getMouseDrag () {
  return RecursiveSlotValue::getSlotValue (this, dragName ());
}
