// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//
#ifndef MouseDragConfiguration_h_included
#include "MouseDragConfiguration.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

class MouseDragConfiguration::MouseDragConfigurationData {
public:
  Point m_start;
  Point m_end;
  Point m_prev;
  MouseDragConfiguration::CommitmentLevel m_commitment;
  MouseDragConfigurationData () {}
  MouseDragConfigurationData (const MouseDragConfigurationData &mdcd) {
    m_start = mdcd.m_start;
    m_end = mdcd.m_end;
    m_prev = mdcd.m_prev;
  }
};

MouseDragConfiguration::MouseDragConfiguration ()
  : m_data (NEW (MouseDragConfigurationData ()))
{}

MouseDragConfiguration::~MouseDragConfiguration () {
  delete (m_data);
  m_data = 0;
}

void MouseDragConfiguration::setCommitment (CommitmentLevel cl) {
  m_data->m_commitment = cl;
}

MouseDragConfiguration::CommitmentLevel MouseDragConfiguration::getCommitment
() const
{
  return m_data->m_commitment;
}

void MouseDragConfiguration::setStart (const Point &p) {
  m_data->m_start = p;
  m_data->m_end = p;
  m_data->m_prev = p;
}

Point MouseDragConfiguration::getStart () const {
  return m_data->m_start;
}

void MouseDragConfiguration::setEnd (const Point &p) {
  m_data->m_prev = m_data->m_end;
  m_data->m_end = p;
}

Point MouseDragConfiguration::getEnd () const {
  return m_data->m_end;
}

Point MouseDragConfiguration::getPrev () const {
  return m_data->m_prev;
}

MouseDragConfiguration::MouseDragConfiguration
(const MouseDragConfiguration &mdc)
  : ActionConfiguration (mdc),
    m_data (NEW (MouseDragConfigurationData (*(mdc.m_data))))
{}

SP<Configuration> MouseDragConfiguration::copy () const {
  return NEW (MouseDragConfiguration (*this));
}

bool MouseDragConfiguration::hasTheSameTypeAs (const Configuration *c) const {
  return 0 != dynamic_cast <const MouseDragConfiguration *> (c);
}
