// This in the public domain.  No copyright.  Tim Freeman  4 Nov 2000.
#ifndef Evaluator_h_included
#include "Evaluator.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef Integrator_h_included
#include "Integrator.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

// For std::cout.
#ifndef iostream_h_included
#include "iostream.h"
#define iostream_h_included
#endif

class Circle: public Evaluator {
  Dynavec<FFloat> m_state;
public:
  Circle () {
    m_state.extendTo (2);
  }
  Dynavec <FFloat> &stateVec () {
    return m_state;
  }
  const Dynavec <FFloat> &stateVec () const {
    return m_state;
  }
  SP<Evaluator::OutOfBounds> eval (FFloat t, const FFloat *state, FFloat *deriv) {
    (void) t;
    deriv [1] = state [0];
    deriv [0] = - state [1];
    return 0;
  }
};

int main (const int argc, const char * const * const argv) {
  if (2 != argc) {
    std::cout << "Usage: test <integrationMethodName>" << std::endl
         << "Such as \"test RungeKuttta\" or \"test AB2\"." << std::endl;
    return 1;
  }
  SP<Configurable> conf = FactoryTable::load ("Integrator", argv[1])->makeIt();
  assert (conf);
  SP<Integrator> rk = dynamic_cast <Integrator *> (&*conf);
  assert (rk);
  SP<Circle> c = NEW (Circle ());
  const FFloat pi = 3.141592653589793;
  const int steps = 40;
  rk->setEvaluator (c);
  rk->start (0.0, 2 * pi / steps);
  c->stateVec()[0] = 0;
  c->stateVec()[1] = 1;
  FFloat time = 0;
  int i = 0;
  for (;;) {
    std::cout << "Time: " << time << " x: "<< c->stateVec()[0]
         << " y: "<< c->stateVec()[1] << std::endl;
    if (i >= steps) break;
    time = rk->observe ();
    i++;
  }
  conf = 0; rk = 0; c = 0;
  MemoryUtil::listLeaks ();
  // The point is that when you integrate all the way around the
  // circle, you should be back where you started. 
  return 0;
}

