// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#include "BallConfiguration.h"

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef Color_h_included
#include "Color.h"
#endif

#ifndef ColorSlotValue_h_included
#include "ColorSlotValue.h"
#endif

#ifndef PositiveFloatSlotValue_h_included
#include "PositiveFloatSlotValue.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

namespace {
  FString materialName () {
    return "Material";
  }

  FString radiusName () {
    return "Radius";
  }
};

BallConfiguration::BallConfiguration ()
{
  initializeSlot (radiusName(), NEW (PositiveFloatSlotValue (1)));
  initializeSlot (materialName (),
		  NEW (ColorSlotValue (Color (0.8, 0.8, 0.8, 1.0))));
}

void BallConfiguration::setMaterial (const Color &material) {
  ColorSlotValue::setSlot (this, materialName(), material);
}

Color BallConfiguration::getMaterial () const {
  return ColorSlotValue::getSlot (this, materialName());
}

void BallConfiguration::setVisibleRadius (FFloat visibleRadius) {
  PositiveFloatSlotValue::setSlot (this, radiusName(), visibleRadius);
}

FFloat BallConfiguration::getVisibleRadius () const {
  return PositiveFloatSlotValue::getSlot (this, radiusName());
}

SP<Configuration> BallConfiguration::copy () const {
  return NEW (BallConfiguration (*this));
}

bool BallConfiguration::hasTheSameTypeAs (const Configuration *c) const {
  return 0 != dynamic_cast <const BallConfiguration *> (c);
}
