// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef BoringBall_h_included
#define BoringBall_h_included

#ifndef Ball_h_included
#include "Ball.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

class BoringBallConfiguration;
class BoundingSphere;
class BoundingBox;

// A Ball with no physics.
class BoringBall: public Ball {
public:
  BoringBall (SP<BoringBallConfiguration> b);
  bool exempt () const;
  BoundingSphere physicsBoundingSphere (const FFloat *state) const;
  bool isPhysicsBounded () const;
  bool physicsIntersect (const FFloat *state, const BoundingBox &box)
    const;
  void plausibleState (Dynavec <FFloat> &result) const;
  FFloat physicsBoundingSphereRadius () const;
  static const char *const staticClassId;
  const char *classId () const;
  bool isInstanceOf (const char *classId) const;
};
  
#endif
