// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef BouncingBall_h_included
#define BouncingBall_h_included

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef Ball_h_included
#include "Ball.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

class BoundingSphere;
class BoundingBox;
class BouncingBallConfiguration;
class Vec3;

#ifndef SP_h_included
#include "SP.h"
#endif

class BouncingBall: public Ball {
  // Mass of the ball.
  const FFloat m_mass;
  // Next constant scales how sharp the physics at the boundary of the
  // ball is.
  // High values make the balls act more like rigid object, at the
  // cost of numerical analysis that is inaccurate without a small
  // step size.
  // Low values give bouncing balls that overlap each other quite a
  // bit before they actually bounce.
  const FFloat m_stiffness;
  // The acceleration of gravity.  A nonnegative number.  Gravity
  // always accelerates in the -z direction.
  const FFloat m_gravity;
  // If two balls are closer than this, then don't try to compute
  // force between them, to avoid divide by 0.
  const FFloat m_smallDistance;
  const bool m_randomizeVelocities;
 public:
  // Not const because Ball updates the conf with billboarding counts. 
  BouncingBall (SP<BouncingBallConfiguration> b);
  ~BouncingBall ();
  void plausibleState (Dynavec <FFloat> &result) const;
  bool exempt () const;
  static inline Vec3 velocity (const FFloat *state) {
    return Vec3 (state + 3);
  }
  static inline void BouncingBall::setVelocity (FFloat *state, const Vec3 &v) {
    v.copyToPointer (state + 3);
  }
  void addVelocity (FFloat *state, const Vec3 &v) const;
  void doPhysics (const FFloat *state, 
		  FFloat *deriv,
		  int myIndex,
		  const Dynavec<PhysicsObjectInfo> &neighbors) const;
  int stateSize () const;
  FFloat physicsBoundingSphereRadius () const;
  bool isPhysicsBounded () const;
  bool physicsIntersect (const FFloat *state, const BoundingBox &box) const;
  static const char *const staticClassId;
  const char *classId () const;
  bool isInstanceOf (const char *classId) const;
};

#endif
