// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Disk_h_included
#define Disk_h_included

#ifndef FFloat_h_included
#include "FFloat.h"
#endif

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

#ifndef Vec3_h_included
#include "Vec3.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

class DiskConfiguration;

class Disk: public PhysicsObject {
  Vec3 m_defaultNorm;
 public:
  Disk (SP<DiskConfiguration> d);
  int stateSize () const;
  void plausibleState (Dynavec <FFloat> &result) const;
  bool physicsIntersect (const FFloat *state, const BoundingBox &box)
    const;

  Vec3 boundingSphereCenter (const FFloat *state) const;
  FFloat physicsBoundingSphereRadius () const; // Aborts.
  bool isPhysicsBounded () const; // returns false.
  
  void translate (FFloat *state, const Vec3 &v) const;
  // Next method does nothing.
  void addVelocity (FFloat *state, const Vec3 &v) const;
  void rotate (FFloat *state, const Quaternion &q) const;
  // The Disk is exempt.
  bool exempt () const;
  static inline Vec3 center (const FFloat *state) {
    return Vec3 (state);
  }
  static inline Vec3 normUnit (const FFloat *state) {
    Vec3 r1 (state + 3);
    assert (0.99 < r1.length() && r1.length() < 1.01);
    return r1;
  }
  static void setCenter (FFloat *state, const Vec3 &v);
  // Next one normalizes v before setting.
  static void setNorm (FFloat *state, const Vec3 &v);
  static const char *const staticClassId;
  const char *classId () const;
  bool isInstanceOf (const char *classId) const;
};

#endif
