// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 N Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef RealSceneSorter_h_included
#include "RealSceneSorter.h"
#endif

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

// Next one needed for qsort.
#ifndef stdlib_h_included
#include <stdlib.h>
#define stdlib_h_included
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

// qsort doesn't pass anything specific about this sort to cmp, so we need to
// put the required information into a global variable.
SP<const SceneGraph> s_sg;

static int cmp (const void *v1, const void *v2) {
  const int i1 = *(const int *)v1;
  const int i2 = *(const int *)v2;
  SP<const PhysicsObject> o1 = s_sg->object(i1);
  SP<const PhysicsObject> o2 = s_sg->object(i2);
  if (o1->lessThan (o2)) {
    return -1;
  } else if (o2->lessThan (o1)) {
    return 1;
  } else if (i1 < i2) {
    // I vaguely recall meeting an implementation of qsort long ago that did
    // funny things when given a funny comparison function.  I don't remember
    // the details, but making the comparison be a total order clearly
    // decreases the total amount of funnyness.  Tim Freeman 18 Oct 2000.
    return -1;
  } else if (i1 > i2) {
    return 1;
  } else {
    return 0;
  }
}

// Permutation will map old indices to new, except old indices of deleted
// objects will be mapped to -1.
void RealSceneSorter::garbageCollectSort (const SceneGraph *sg,
                                          Dynavec<int> &permutation,
                                          int &newLength) {
  // Inverse will map new indices to old.
  Dynavec <int> inverse;
  const int sourceLength = sg->maxIndex();
  for (int i = 0; i < sourceLength; i++) {
    if (sg->object (i)) {
      inverse.push (i);
    }
  }
  s_sg = sg;
  // Now we're sorting the old indices.
  qsort ((void *) (&*inverse), inverse.size(),
         sizeof (int), cmp);
  s_sg = 0;
  newLength = inverse.size ();
  permutation.extendTo (sg->maxIndex());
  for (int i = 0; i < sourceLength; i++) {
    permutation [i] = -1;
  }
  for (int i = 0; i < newLength; i++) {
    permutation [inverse [i]] = i;
  }
}
