// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef SubsetStorageConfiguration_h_included
#define SubsetStorageConfiguration_h_included

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef SubsetStorage_h_included
#include "SubsetStorage.h"
#endif

class RecursiveSlotValue;

class SubsetStorageConfiguration : public Configuration {
  SubsetStorage::Maintainer * m_maintainer;
  // The number of times that we went to move an object and found
  // that it had gone outside of its approximate bounding box. 
  int m_moved;
  // The number of times that we went to move an object and found that
  // it had not gone outside of its approximate bounding box.
  int m_notMoved;
public:
  SubsetStorageConfiguration ();
  SubsetStorageConfiguration (const SubsetStorageConfiguration &);
  // Tweak the next three so they updated and read m_moved and m_notMoved.
  SP<BadSlotValue> setSlot (const FString &name, SP<SlotValue> sv);
  SP<const SlotValue> getSlot (const FString &name) const;
  SP<SlotValue> getSlot (const FString &name);
  SP<const RecursiveSlotValue> getIncrementalStorageSlot () const;
  SP<RecursiveSlotValue> getIncrementalStorageSlot ();
  void setIncrementalStorageSlot (SP<RecursiveSlotValue> rsv);
  SubsetStorage::Maintainer *getMaintainer () const;
  void setMaintainer (SubsetStorage::Maintainer *m);
  SP<Configuration> copy () const;
  bool hasTheSameTypeAs (const Configuration *c) const;
  inline void incMoved () {
    m_moved++;
  }
  inline void incNotMoved () {
    m_notMoved++;
  }
};

#endif
