// Fungimol - an extensible system for designing atomic-scale objects.
// Copyright (C) 2000 Tim Freeman
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library in the file COPYING.txt; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place -
// Suite 330, Boston, MA 02111-1307, USA
//
// The author can be reached by email at tim@fungible.com, or by
// paper mail at:
//
// Tim Freeman
// 381 North Fernwood Circle
// Sunnyvale, CA 94085-3059
//

#ifndef Factory_h_included
#include "Factory.h"
#endif

#ifndef MemoryUtil_h_included
#include "MemoryUtil.h"
#endif

#ifndef SP_h_included
#include "SP.h"
#endif

#ifndef Configurable_h_included
#include "Configurable.h"
#endif

#ifndef Configuration_h_included
#include "Configuration.h"
#endif

#ifndef ActionConfiguration_h_included
#include "ActionConfiguration.h"
#endif

#ifndef myassert_h_included
#include "myassert.h"
#endif

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef TopLevel_h_included
#include "TopLevel.h"
#endif

#ifndef SelectionManager_h_included
#include "SelectionManager.h"
#endif

#ifndef SceneGraph_h_included
#include "SceneGraph.h"
#endif

#ifndef FString_h_included
#include "FString.h"
#endif

#ifndef Action_h_included
#include "Action.h"
#endif

#ifndef FactoryTable_h_included
#include "FactoryTable.h"
#endif

#ifndef LinkManager_h_included
#include "LinkManager.h"
#endif

#ifndef PhysicsObject_h_included
#include "PhysicsObject.h"
#endif

namespace {
  // This code is more-or-less obsolete.  We really want to be able to toggle
  // whether the current molecule is selected.  That requires one routine to
  // see the mouse click and then toggle the selection on that molecule.  In
  // contrast, this action assumes that some previous code selects the atom
  // under the mouse, and this code then selects the molecule.  This can't be
  // slightly tweaked to do the right thing because if the molecule is small,
  // then when the atom in the molecule is selected, we lose all information
  // about whether the molecule was selected before.
  // However, before deleting this code find all uses of it and think a bit.
  // The replacement action is ToggleConnectedSelection.
  class GrowSelectionConnected
    : public Factory
  {
  public:
    GrowSelectionConnected ()
      : Factory ("GrowSelectionConnected")
    {}
    SP<Configuration> defaultConfiguration () const {
      return NEW (ActionConfiguration ());
    }
    SP<Configurable> makeIt (const SP<Configuration> c) const {
      SP<ActionConfiguration> conf =
	dynamic_cast <ActionConfiguration *> (&*c);
      assert (conf);
      Dynavec <int> selection;
      SelectionManager::getSelection (selection);
      SP<Action> result = NEW (Action ());
      if (0 == selection.size()) {
	result->setProblem ("Select something first.");
      }
      if (!result->isProblem ()) {
	SP<TopLevel> top = conf->getTopLevel ();
	SP<SceneGraph> sg = top->getSceneGraph ();
	SP<LinkManager> lm = sg->getLinkManager ();
	// Set of atoms that we'll select at the end.  This is where we
	// accumulate the result.
	Dynavec <bool> selectionSet;
	selectionSet.extendTo (sg->maxIndex (), false);
	// This could logically go inside the loop, except I don't want
	// redundant memory allocation.
	Dynavec <int> newSelection;
	for (;;) {
	  bool anyAdded = false;
	  newSelection.extendTo (0);
	  // selection is our work queue.  newSelection is what we'll work on
	  // next time around the loop.  If any elements in selection aren't
	  // already in the result, put them in the result and put their
	  // neighbors in newSelection.
	  for (int i = 0; i < selection.size (); i++) {
	    const int s = selection [i];
	    // Note the call to PhysicsObject::isSelectable rather than
	    // SelectionManager::isSelectable.  We probably want to select
	    // invisible object here, since they are likely to have the proper
	    // connections.
	    if (!selectionSet [s] && sg->object(s)->isSelectable ()) {
	      selectionSet [s] = true;
	      anyAdded = true;
	      int count;
	      const int *links;
	      lm->getLinks (s, count, links);
	      for (int i = 0; i < count ; i++) {
		const int thisLink = links [i];
		if (-1 != thisLink) {
		  newSelection.push (thisLink);
		}
	      }
	    }
	  }
	  selection = newSelection;
	  if (!anyAdded) break;
	}
	// Convert selectionSet to a list, sacraficing selection, which we no
	// longer have any use for.
	selection.extendTo (0);
	for (int i = 0; i < selectionSet.size(); i++) {
	  if (selectionSet [i]) {
	    selection.push (i);
	  }
	}
	SelectionManager::setSelection (sg, selection);
      }
      return &*result;
    }
  };
  const bool useless =
  (FactoryTable::store ("Action", NEW (GrowSelectionConnected ())),
   true);
}
