#ifndef State_h_included
#define State_h_included

// This is the definition of State for the Fungimol interface.

#ifndef Dynavec_h_included
#include "Dynavec.h"
#endif

#ifndef bvector_h_included
#include "bvector.h"
#endif

class SceneGraphPOInterface;
class NeighborState;
struct State {
  // A State is a physical configuration of the BrennerAtom's in the system.
  // 
  // This pointer is owned by the scene graph, so it is allowed to dangle when
  // we aren't in the process of computing physics. 
  const SceneGraphPOInterface *sg;
  // state is the present position and velocity given by the integrator.
  // 
  // This pointer is owned by the scene graph, so it is allowed to dangle when
  // we aren't in the process of computing physics. 
  const Float *state; 
  // The computed velocity and acceleration should be added into
  // deriv. The velocity is the derivative of the position, so we eventually
  // have to copy the velocity out of state into the deriv vector at the
  // same index as the position coordinates in state.
  // 
  // This pointer is owned by the scene graph, so it is allowed to dangle when
  // we aren't in the process of computing physics. 
  Float *deriv;
  Float rll;
private:
  NeighborState *m_ljNeighbors;
  NeighborState *m_caNeighbors;
  int firstIndex;
  int pastLastIndex;
  // forceMiddle is what we really allocate.
  // forceStart is what we index from.  forceStart + firstIndex = forceMiddle.
  vector *forceMiddle;
  vector *forceStart;
  vector *prevAtomStart;
  vector *prevAtomMiddle;
public:
  inline NeighborState *ljNeighbors () {
    return m_ljNeighbors;
  }
  inline const NeighborState *ljNeighbors () const {
    return m_ljNeighbors;
  }
  inline NeighborState *caNeighbors () {
    return m_caNeighbors;
  }
  inline const NeighborState *caNeighbors () const {
    return m_caNeighbors;
  }
  inline vector &force(int i) {
    return forceStart [i];
  }
  inline const vector &force(int i) const {
    return forceStart [i];
  }
  inline vector &prevAtom (int i) {
    return prevAtomStart [i];
  }
  inline const vector &prevAtom (int i) const {
    return prevAtomStart [i];
  }
  inline int getFirstIndex () const {
    return firstIndex;
  }
  inline int getPastLastIndex () const {
    return pastLastIndex;
  }
  // The number of atoms of each ktype.
  Dynavec<int> noa;
  // start is the first index we're going to use; should be nonnegative.  
  // end is the last index we're going to use.  Call setRange to set
  // firstIndex, pastLastIndex, forceStart, and forceMiddle.
  void setRange (int start, int end);
  // The constructor initializes forceMiddle to zero.
  State ();
  // The destructor frees forceMiddle if it is nonzero.
  ~State ();
};


#endif
