/* xmass.h - Copyright (c) 1998 Zyvex LLC.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef XMASS_h_included
#define XMASS_h_included

/* Need kt.h for NTYPES. */
#ifndef KT_h_included
#include "kt.h"
#endif

/* BOLZ is only used to compute ECONV. */
#define BOLZ 1.380662
/* EPSI0 is only used to compute ECONV. */
#define EPSI0 11604.5 /* EPSI as initialized in setin.f */
/* EPSI is only used in bren2.c when dealing with temperatures. */
#define EPSI 11605.0
/* AVO is only used to compute ECONV. */
#define AVO 6.02205
/* To convert the mass of an atom (as given in the xmass array) into real
   units, multiply by ECONV. */
#define ECONV ((1/(AVO*BOLZ*EPSI0))*1e7)

/* If an atom has atomic number n, then xmass [kt [n]] is the mass of one of
   them.  This is given a default value in alloc_bren in bren2.c, and it may
   be given a non-default value when reading input.d */
extern Float xmass[NTYPES+1];
/* emass is like xmass, except each element is multiplied by ECONV. */
extern Float emass[NTYPES+1];

/* You'd better call this before using xmass or emass. */
void static_initxmass ();
#endif
